package chess.mobile.engineoutput;

import android.util.Log;

import org.greenrobot.eventbus.EventBus;

import java.util.Observable;
import java.util.Observer;

import chess.mobile.Uci;
import chess.mobile.chessgame.ChessGameUtils;
import chess.mobile.chessgame.ObservableChessGame;
import chess.mobile.dao.EngineDAO;
import chess.mobile.dao.UserSettingsDAO;
import chess.mobile.event.ConnectToWebSocketEvent;
import chess.mobile.event.DisconnectFromWebSocketEvent;
import chess.mobile.event.SendMessageEvent;
import chess.mobile.dao.manager.EngineManager;
import chess.mobile.model.UciInfo;
import chess.mobile.rest.Response;
import chess.mobile.rest.ResponseListener;

/**
 * Created by Stanisław Kabaciński.
 */
public class EngineOutputPresenter implements EngineOutputContract.Presenter, Observer {

    private static final String TAG = "EnginePresenter";
    private EngineOutputContract.View view;
    private EngineDAO engineDAO;
    private ObservableChessGame chessGame;
    private UserSettingsDAO userSettingsDAO;
    private boolean engineRunning = false;
    private volatile boolean startPending = false;

    public EngineOutputPresenter(EngineOutputContract.View view, EngineDAO engineDAO, ObservableChessGame chessGame, UserSettingsDAO userSettingsDAO) {
        this.engineDAO = engineDAO;
        this.chessGame = chessGame;
        this.view = view;
        this.userSettingsDAO = userSettingsDAO;
        this.view.setPresenter(this);
    }

    @Override
    public void stop() {
        chessGame.deleteObserver(this);
        Log.d(TAG, "stop");
        EventBus.getDefault().post(new DisconnectFromWebSocketEvent());
    }

    @Override
    public void start() {
        chessGame.addObserver(this);
        view.showOutput("");
        view.showNoEngineSelected();
        Log.d(TAG, "start");
        EventBus.getDefault().post(new ConnectToWebSocketEvent(userSettingsDAO.getUrl().replace("http", "ws") + "/ws_engine"));
    }

    @Override
    public void onEngineChanged(String engineName) {
        EngineManager.getInstance().setEngineName(engineName);
        view.setEngineName(engineName);
        view.showOutput("");
        view.showEngineBooting();
        engineRunning = false;
        startPending = false;
        engineDAO.startEngine(engineName, new ResponseListener<Response>() {
            @Override
            public void onRequestFailure(Exception e) {
            }

            @Override
            public void onRequestSuccess(Response response) {
                EventBus.getDefault().post(new SendMessageEvent("uci"));
            }
        });
    }

    @Override
    public void onMessageReceived(String message) {
        if (message.equalsIgnoreCase(Uci.UCI_OK) || message.startsWith(Uci.BEST_MOVE)) {
            view.showEngineIdle();

            synchronized (this) {
                if (startPending) {
                    run();
                    startPending = false;
                }
            }
        }
    }

    @Override
    public void onInfoReceived(UciInfo info) {
        if (view.isActive()) {
           view.showDepthAndEvaluation(info.cp, info.depth);
            if (info.pv != null) {
                view.showOutput(info.pv);
            }
        }
    }

    @Override
    public void run() {
        if (EngineManager.getInstance().isReady()) {
            engineRunning = true;
            EventBus.getDefault().post(new SendMessageEvent("position startpos moves " + ChessGameUtils.getChessGameMovesAsPAN(chessGame)));
            EventBus.getDefault().post(new SendMessageEvent("go infinite"));
            view.showEngineRunning();
        } else {
            view.showEngineNotReadyError();
        }
    }

    @Override
    public void pause() {
        if (engineRunning) {
            engineRunning = false;
            EventBus.getDefault().post(new SendMessageEvent("stop"));
        }
    }

    @Override
    public void showOptions() {
        if (EngineManager.getInstance().isReady() && !engineRunning) {
            view.showOptions();
        } else {
            view.showOptionsNotReadyError();
        }
    }

    @Override
    public void update(Observable o, Object arg) {
        if (arg.equals(ObservableChessGame.ACTION_NEW_GAME)
                || arg.equals(ObservableChessGame.ACTION_NEW_MOVE)
                || arg.equals(ObservableChessGame.ACTION_UNDO)
                && engineRunning) {
            synchronized (this) {
                startPending = true;
                pause();
            }
        }
    }
}
