package chess.mobile.game;

import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.app.AlertDialog;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import java.util.Locale;
import java.util.Observable;
import java.util.Observer;

import butterknife.ButterKnife;
import chess.mobile.R;
import chess.mobile.chessgame.ChessGame;
import chess.mobile.chessgame.ChessGameImpl;
import chess.mobile.chessgame.Move;
import chess.mobile.chessgame.MoveFactory;
import chess.mobile.chessgame.ObservableChessGame;
import chess.mobile.chessgame.Piece;
import chess.mobile.chessgame.PieceColor;
import chess.mobile.chessgame.PossibleMovesProvider;
import chess.mobile.chessgame.PossibleMovesProviderImpl;
import chess.mobile.chessgame.SANHelper;
import chess.mobile.view.chess_view.ChessboardView;

import static butterknife.ButterKnife.findById;

/**
 * Created by Stanisław Kabaciński.
 */

public class BoardFragment extends Fragment implements BoardFragmentContract.View, Observer {

    private BoardFragmentContract.Presenter presenter;

    private MoveFactory moveFactory;
    private ObservableChessGame chessGame;
    private ChessboardView chessboardView;
    private TextView output;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
    }

    @Override
    public void onResume() {
        super.onResume();
        presenter.start();
        if (chessGame != null) {
            chessGame.addObserver(this);
        }
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        inflater.inflate(R.menu.board_fragment_menu, menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.swap_sides) {
            if (chessboardView != null && this.getView() != null) {
                chessboardView.setInverted(!chessboardView.isInverted());
                return true;
            }
        }
        return false;
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View v = inflater.inflate(R.layout.fragment_chess_game, container, false);
        ButterKnife.bind(this, v);
        chessboardView = findById(v, R.id.chessboard);
        output = findById(v, R.id.output);
        setChessGame(chessGame == null ? chessGame = new ObservableChessGame() : chessGame);
        chessboardView.setOnMoveListener((fromX, fromY, toX, toY) -> {
            Move move = moveFactory.getMove(chessGame.getBoard(), fromX, fromY, toX, toY);
            if (move.getPromotion() == 0) {
                chessGame.makeMove(move);
            }
        });
        return v;
    }

    public void setChessGame(ObservableChessGame chessGame) {
        this.chessGame = chessGame;
        this.chessGame.addObserver(this);
        if (chessboardView != null) {
            setupChessGame(chessGame);
        }
    }

    @Override
    public void onPause() {
        super.onPause();
        if (chessGame != null) {
            chessGame.deleteObserver(this);
        }
    }

    private void setupChessGame(ObservableChessGame chessGame) {
        chessboardView.setBoard(chessGame.getChessGame().getBoard());
        PossibleMovesProvider movesProvider = new PossibleMovesProviderImpl(chessGame);
        chessboardView.setPossibleMovesProvider(movesProvider);
        moveFactory = new MoveFactory(movesProvider);
        moveFactory.setOnMoveMadeListener(this::showPromotionDialog);
        chessboardView.resetSelection();
    }

    private void showPromotionDialog(Move move) {
        new AlertDialog.Builder(getContext())
                .setTitle(R.string.promotion_dialog_title)
                .setCancelable(false)
                .setItems(
                        move.getColor() == PieceColor.BLACK ? R.array.black_promotion_pieces : R.array.white_promotion_pieces,
                        (dialog, which) -> {
                            switch (which) {
                                case 0:
                                    move.setPromotion(Piece.QUEEN);
                                    break;
                                case 1:
                                    move.setPromotion(Piece.KNIGHT);
                                    break;
                                case 2:
                                    move.setPromotion(Piece.QS_ROOK);
                                    break;
                                case 3:
                                    move.setPromotion(Piece.BISHOP);
                                    break;
                            }
                            if (move.getPromotion() != 0) {
                                move.setSan(move.getSan() + "=" + SANHelper.getSymbol(move.getPromotion() + move.getColor()));
                                chessGame.getBoard()[move.getToX()][move.getToY()] = move.getPromotion() + move.getColor();
                            }
                            chessGame.makeMove(move);
                        })
                .create().show();
    }


    @Override
    public void showNoEngineOutput() {
        output.setText(R.string.no_data_to_display);
    }

    @Override
    public void showLastEngineLine(String output, Integer depth, Integer cp) {
        String out = "";
        if (cp != null) {
            float eval = cp / 100.f;
            out += "[" + (eval > 0 ? "+" : "") + String.format(Locale.US, "%.2f", eval) + "] ";
        }
        if (depth != null) {
            out += "d=" + depth;
        }
        final String txt = out;
        getActivity().runOnUiThread(() -> {
            this.output.setText(txt + " " + Html.fromHtml(output));
        });
    }

    @Override
    public void setPresenter(BoardFragmentContract.Presenter presenter) {
        this.presenter = presenter;
    }

    @Override
    public void update(Observable observable, Object o) {
        if (o.equals(ObservableChessGame.ACTION_NEW_GAME)) {
            setupChessGame(chessGame);
        }
    }
}
