package chess.mobile.moveslist;

import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import java.util.List;

import chess.mobile.R;
import chess.mobile.chessgame.Comment;
import chess.mobile.chessgame.Entity;
import chess.mobile.chessgame.Move;
import chess.mobile.chessgame.VariantBegin;
import chess.mobile.chessgame.VariantEnd;
import chess.mobile.util.OnItemClickListener;

/**
 * Created by Stanisław Kabaciński.
 */
class MovesListAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> implements OnItemClickListener<Entity> {

    private static int TYPE_GAME_BEGIN = 0;
    private static int TYPE_MOVE = 1;
    private static int TYPE_COMMENT = 2;
    private static int TYPE_VARIANT = 3;

    private OnItemClickListener<Entity> onItemClickListener;
    private List<Entity> entities;
    private int selected = -1;

    MovesListAdapter(OnItemClickListener<Entity> onItemClickListener, List<Entity> movesList) {
        this.onItemClickListener = onItemClickListener;
        this.entities = movesList;
    }

    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        LayoutInflater inflater = LayoutInflater.from(parent.getContext());
        if (viewType == TYPE_GAME_BEGIN) {
            return new GameBeginViewHolder(inflater.inflate(R.layout.item_comment, parent, false));
        } else if (viewType == TYPE_MOVE) {
            return new MoveViewHolder(inflater.inflate(R.layout.item_move, parent, false));
        } else if (viewType == TYPE_COMMENT) {
            return new CommentViewHolder(inflater.inflate(R.layout.item_comment, parent, false));
        } else if (viewType == TYPE_VARIANT) {
            return new VariantViewHolder(inflater.inflate(R.layout.item_variant, parent, false));
        }
        return null;
    }

    @Override
    public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
        Entity entity = entities.get(holder.getAdapterPosition());
        int depth = getDepth(holder.getAdapterPosition());
        if (holder instanceof MoveViewHolder) {
            ((MoveViewHolder) holder).bind(((Move) entity), this, depth,
                    getMoveString(holder.getAdapterPosition()), selected == holder.getAdapterPosition());
        } else if (holder instanceof CommentViewHolder) {
            ((CommentViewHolder) holder).bind(((Comment) entity), this, depth);
        } else if (holder instanceof VariantViewHolder) {
            VariantViewHolder h = (VariantViewHolder) holder;
            boolean isBegin = getItem(h.getAdapterPosition()) instanceof VariantBegin;
            h.bind(isBegin ? VariantViewHolder.TYPE_BEGIN : VariantViewHolder.TYPE_END, this, depth, entity, selected == holder.getAdapterPosition());
        } else if (holder instanceof GameBeginViewHolder) {
            GameBeginViewHolder gameBegin = ((GameBeginViewHolder) holder);
            gameBegin.bind(this, depth, selected == holder.getAdapterPosition());
        }
    }

    private String getMoveString(int position) {
        Entity e = getItem(position);
        if (!(e instanceof Move)) {
            return "";
        }
        Move m = ((Move) e);
        int moveNo = m.getNo();
        if (moveNo % 2 == 1) {
            return moveNo / 2 + 1 + ".";
        } else if (position > 0 && !(entities.get(position - 1) instanceof Move)) {
            return moveNo / 2 + "…";
        }
        return "";
    }

    @Override
    public int getItemViewType(int position) {
        Entity e = getItem(position);
        if (e instanceof Comment) {
            return TYPE_COMMENT;
        } else if (e instanceof Move) {
            return TYPE_MOVE;
        } else if (e instanceof VariantBegin || e instanceof VariantEnd) {
            return TYPE_VARIANT;
        }
        return 0;
    }

    public List<Entity> getMovesList() {
        return entities;
    }

    public void setMovesList(List<Entity> movesList) {
        this.entities = movesList;
    }

    private Entity getItem(int position) {
        return entities.get(position);
    }

    @Override
    public int getItemCount() {
        if (entities == null) return 0;
        return entities.size();
    }

    public int getSelectedIndex() {
        return selected;
    }

    private int getDepth(int position) {
        int depth = 0;
        for (int i = 0; i <= position; i++) {
            if (entities.get(i) instanceof VariantBegin) {
                depth++;
            } else if (entities.get(i) instanceof VariantEnd) {
                depth--;
                if (position == i) {
                    depth++;
                }
            }
        }
        return depth;
    }

    @Override
    public void onItemClick(View v, int position, Entity object) {
        if (v.getId() != R.id.button) {
            if (selected == position) {
                selected = -1;
                notifyItemChanged(position);
            } else if (!(object instanceof Comment)) {
                notifyItemChanged(selected);
                notifyItemChanged(position);
                selected = position;
            }
        }
        if (onItemClickListener != null) {
            onItemClickListener.onItemClick(v, position, object);
        }
    }

    public void setSelectedIndex(int selectedIndex) {
        int prevSelected = this.selected;
        this.selected = selectedIndex;
        if (inRange(0, entities.size(), prevSelected)) {
            notifyItemChanged(prevSelected);
        }
        if (inRange(0, entities.size(), selected)) {
            notifyItemChanged(selected);
        }
    }

    private static boolean inRange(int lower, int upper, int value) {
        return value >= lower && value < upper;
    }
}
