package chess.mobile.moveslist;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.PopupMenu;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import chess.mobile.R;
import chess.mobile.chessgame.Comment;
import chess.mobile.chessgame.Entity;
import chess.mobile.chessgame.GameBegin;
import chess.mobile.chessgame.Move;
import chess.mobile.chessgame.NumericAnnotationGlyph;
import chess.mobile.chessgame.VariantBegin;
import chess.mobile.chessgame.VariantEnd;
import chess.mobile.util.ListUtils;
import chess.mobile.util.OnItemClickListener;

/**
 * Created by Stanisław Kabaciński.
 */

public class MovesListFragment extends Fragment implements MovesListContract.View, OnItemClickListener<Entity> {

    private static final String TAG = MovesListFragment.class.getSimpleName();
    private static final int REQUEST_ADD_COMMENT = 456;
    private static final int REQUEST_EDIT_COMMENT = 475;
    private static final String BUNDLE_SELECTED_INDEX = "sel_idx";

    @BindView(R.id.recycler_view) RecyclerView recyclerView;
    private MovesListAdapter adapter;
    private MovesListContract.Presenter presenter;

    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt(BUNDLE_SELECTED_INDEX, presenter.getSelectedIndex());
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View v = inflater.inflate(R.layout.fragment_moves_list, container, false);
        ButterKnife.bind(this, v);
        adapter = new MovesListAdapter(this, new ArrayList<>(0));
        this.recyclerView.setAdapter(adapter);
        this.recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        if (savedInstanceState != null && savedInstanceState.containsKey(BUNDLE_SELECTED_INDEX)) {
            presenter.setSelectedIndex(savedInstanceState.getInt(BUNDLE_SELECTED_INDEX));
        }
        return v;
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Activity.RESULT_OK) {
            if (requestCode == REQUEST_ADD_COMMENT) {
                String value = data.getStringExtra(CommentDialog.KEY_COMMENT_VALUE);
                int position = data.getIntExtra(CommentDialog.KEY_POSITION, adapter.getItemCount() - 1);
                presenter.onCommentAdded(value, position);
            } else if (requestCode == REQUEST_EDIT_COMMENT) {
                boolean remove = data.getBooleanExtra(CommentDialog.KEY_REMOVED, false);
                int position = data.getIntExtra(CommentDialog.KEY_POSITION, adapter.getItemCount() - 1);
                if (remove) {
                    presenter.removeComment(position);
                } else {
                    String value = data.getStringExtra(CommentDialog.KEY_COMMENT_VALUE);
                    presenter.onCommentEdited(value, position);
                }
            }
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        presenter.start();
    }

    @Override
    public void onPause() {
        super.onPause();
        presenter.stop();
    }

    @Override
    public void setMovesList(List<Entity> movesList) {
        adapter.setMovesList(movesList);
        adapter.notifyDataSetChanged();
    }

    @Override
    public void addNewMove(Entity entity) {
        adapter.getMovesList().add(entity);
        adapter.notifyItemInserted(adapter.getItemCount());
        recyclerView.post(() -> recyclerView.getLayoutManager().scrollToPosition(adapter.getItemCount() - 1));
    }

    @Override
    public void undoLastMove() {
        adapter.getMovesList().remove(adapter.getMovesList().size() - 1);
        adapter.notifyItemRemoved(adapter.getItemCount());
        recyclerView.post(() -> recyclerView.getLayoutManager().scrollToPosition(adapter.getItemCount() - 1));
    }

    @Override
    public void showDetailsDialog(Entity move, int moveNo) {

    }

    @Override
    public void updateMove(int position) {
        adapter.notifyItemChanged(position);
    }

    @Override
    public void notifyItemInserted(int position) {
        adapter.notifyItemInserted(position);
    }

    @Override
    public void notifyItemRemoved(int position) {
        adapter.notifyItemRemoved(position);
    }

    @Override
    public void notifyItemChanged(int position) {
        adapter.notifyItemChanged(position);
    }

    @Override
    public int getSelectedIndex() {
        return adapter.getSelectedIndex();
    }

    @Override
    public void setSelectedIndex(int selectedIndex) {
        adapter.setSelectedIndex(selectedIndex);
        recyclerView.scrollToPosition(selectedIndex);
    }

    @Override
    public void clearSelection() {
        adapter.setSelectedIndex(-1);
    }

    @Override
    public void showAddComment(int position) {
        Bundle args = new Bundle();
        args.putInt(CommentDialog.KEY_POSITION, position);
        CommentDialog commentDialog = new CommentDialog();
        commentDialog.setArguments(args);
        commentDialog.setTargetFragment(this, REQUEST_ADD_COMMENT);
        commentDialog.show(getFragmentManager(), commentDialog.getClass().getSimpleName());
    }

    @Override
    public void showEditComment(int position, Comment comment) {
        Bundle args = new Bundle();
        args.putInt(CommentDialog.KEY_POSITION, position);
        args.putString(CommentDialog.KEY_COMMENT_VALUE, comment.getValue());
        CommentDialog commentDialog = new CommentDialog();
        commentDialog.setArguments(args);
        commentDialog.setTargetFragment(this, REQUEST_EDIT_COMMENT);
        commentDialog.show(getFragmentManager(), commentDialog.getClass().getSimpleName());
    }

    @Override
    public void setPresenter(MovesListContract.Presenter presenter) {
        this.presenter = presenter;
    }

    @Override
    public void onItemClick(View v, int position, Entity object) {
        if (v.getId() == R.id.button) {
            PopupMenu popup = new PopupMenu(getContext(), v);

            popup.getMenuInflater().inflate(R.menu.popup_comment, popup.getMenu());

            if (object instanceof VariantBegin || object instanceof VariantEnd) {
                popup.getMenuInflater().inflate(R.menu.popup_variant, popup.getMenu());
            } else if (object instanceof Move) {
                if (presenter.canDeleteMove(position)) {
                    popup.getMenuInflater().inflate(R.menu.popup_move_delete, popup.getMenu());
                }
                popup.getMenuInflater().inflate(R.menu.popup_move_base, popup.getMenu());
            } else if (object instanceof Comment) {
                popup.getMenuInflater().inflate(R.menu.popup_comment_delete, popup.getMenu());
            }

            popup.setOnMenuItemClickListener(item -> {
                switch (item.getItemId()) {
                    case R.id.add_comment:
                        showAddComment(position + 1);
                        return true;
                    case R.id.comment_delete:
                        presenter.removeComment(position);
                        return true;
                    case R.id.delete_move:
                        presenter.removeMove(position);
                        return true;
                    case R.id.edit_nag:
                        presenter.onSelectGlyphClicked(position);
                        return true;
                    case R.id.delete_variant:
                        presenter.removeSubVariant(position, object);
                        return true;
                    default:
                        return false;
                }
            });
            popup.show();
        } else {
            if (object instanceof Move || object == null) {
                presenter.onMoveClicked(position);
            } else if (object instanceof Comment) {
                presenter.onCommentClicked(position, ((Comment) object));
            }
        }
    }

    @Override
    public void showEditGlyph(int position, int[] nags) {
        List<NumericAnnotationGlyph> glyphs = NumericAnnotationGlyph.getGlyphs();
        String[] array = new String[glyphs.size()];
        for (int i = 0; i < array.length; i++) {
            array[i] = glyphs.get(i).string + " ($" + glyphs.get(i).no + ")";
        }

        boolean selected[] = new boolean[glyphs.size()];
        for (int i = 0; i < selected.length; i++) {
            selected[i] = false;
            for (int nag : nags) {
                if (glyphs.get(i).no == nag) {
                    selected[i] = true;
                }
            }
        }

        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder.setTitle(R.string.edit_nag)
                .setMultiChoiceItems(array, selected, (dialog, which, isChecked) -> {
                    selected[which] = isChecked;
                })
                .setPositiveButton(R.string.ok, (dialog, i) -> {
                    List<Integer> nagsList = new ArrayList<>();
                    for (int j = 0; j < selected.length; j++) {
                        if (selected[j]) {
                            nagsList.add(glyphs.get(j).no);
                        }
                    }
                    presenter.onGlyphChanged(position, ListUtils.intListToArray(nagsList));
                    dialog.dismiss();
                })
                .setNegativeButton(R.string.cancel, (dialog, i) -> {
                    dialog.dismiss();
                })
                .show();
    }

    @Override
    public void notifyDataSetChanged() {
        adapter.notifyDataSetChanged();
    }
}
