package chess.mobile.moveslist;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;

import chess.mobile.chessgame.ChessGame;
import chess.mobile.chessgame.ChessGameImpl;
import chess.mobile.chessgame.Comment;
import chess.mobile.chessgame.Entity;
import chess.mobile.chessgame.GameBegin;
import chess.mobile.chessgame.Move;
import chess.mobile.chessgame.MovesFilter;
import chess.mobile.chessgame.ObservableChessGame;
import chess.mobile.chessgame.VariantBegin;
import chess.mobile.chessgame.VariantEnd;

/**
 * Created by Stanisław Kabaciński.
 */

public class MovesListPresenter implements MovesListContract.Presenter {

    private MovesListContract.View view;
    private ObservableChessGame chessGame;
    private List<Entity> entities;
    private boolean moveUpdatesLocked = false;
    private int selectedIndex = 0;

    public MovesListPresenter(MovesListContract.View view, ObservableChessGame chessGame) {
        this.view = view;
        this.view.setPresenter(this);
        this.chessGame = chessGame;
        this.entities = new ArrayList<>();
    }

    @Override
    public void stop() {
        chessGame.deleteObserver(this);
    }

    @Override
    public void start() {
        chessGame.addObserver(this);
        view.setMovesList(entities);
        if (entities.size() > 0) {
            onMoveSelected(selectedIndex);
            view.setSelectedIndex(selectedIndex);
        }
    }

    @Override
    public void update(Observable o, Object arg) {
        if (moveUpdatesLocked) return;

        if (arg.equals(ObservableChessGame.ACTION_NEW_GAME)) {
            entities = new ArrayList<>();
            entities.add(new GameBegin());
            view.setMovesList(entities);
        } else if (arg.equals(ObservableChessGame.ACTION_NEW_MOVE)) {
            Move move = chessGame.getMoveList().get(chessGame.getMoveList().size() - 1);
            insertMove(move);
            view.notifyDataSetChanged();
        } else if (arg.equals(ObservableChessGame.ACTION_UNDO)) {
            view.undoLastMove();
        }
    }

    @Override
    public void selectNextMove() {
        if (entities.size() == 0) {
            return;
        }
        if (selectedIndex < 0) {
            selectedIndex = entities.size() - 1;
        } else {
            selectedIndex++;
            if (selectedIndex >= entities.size()) {
                selectedIndex = entities.size() - 1;
            }
        }

        //skip variants
        if (entities.get(selectedIndex) instanceof VariantBegin) {
            int inVariantCount = 1;

            for (int i = selectedIndex + 1; i < entities.size(); i++) {
                Entity e = entities.get(i);
                if (e instanceof VariantBegin) {
                    inVariantCount++;
                } else if (e instanceof VariantEnd) {
                    inVariantCount--;
                } else if (inVariantCount == 0 && e instanceof Move) {
                    selectedIndex = i;
                    break;
                }
            }
        }
        while (entities.get(selectedIndex) instanceof VariantEnd) {
            selectedIndex--;
        }

        onMoveSelected(selectedIndex);
        view.setSelectedIndex(selectedIndex);
    }

    @Override
    public void selectPreviousMove() {
        if (entities.size() == 0) {
            return;
        }
        if (selectedIndex < 0) {
            selectedIndex = entities.size() - 1;
        }
        selectedIndex--;
        if (selectedIndex < 0) {
            selectedIndex = 0;
        }

        // skip variants
        if (entities.get(selectedIndex) instanceof VariantEnd) {
            int inVariantCount = 1;

            for (int i = selectedIndex - 1; i >= 0; i--) {
                Entity e = entities.get(i);
                if (e instanceof VariantBegin) {
                    inVariantCount--;
                } else if (e instanceof VariantEnd) {
                    inVariantCount++;
                } else if (inVariantCount == 0 && e instanceof Move) {
                    selectedIndex = i;
                    break;
                }
            }
        }
        while (entities.get(selectedIndex) instanceof VariantBegin) {
            selectedIndex--;
        }

        onMoveSelected(selectedIndex);
        view.setSelectedIndex(selectedIndex);
    }

    private void insertMove(Move m) {
        int position = -1;
        boolean createSubVariant = false;
        int inVariantCount = 0;

        //checks if next move is the same as the one performed
        if (entities.size() > selectedIndex + 1) {
            Entity next = entities.get(selectedIndex + 1);
            if (next instanceof Move && next.equals(m)) {
                selectedIndex++;
                view.setSelectedIndex(selectedIndex);
                return;
            }
        }

        //checks if in following variants any first move matches performed move
        if (entities.size() > selectedIndex + 2 && !(entities.get(selectedIndex + 1) instanceof VariantEnd)) {
            if (entities.get(selectedIndex + 1) instanceof VariantBegin) {
                inVariantCount++;
            }
            for (int i = selectedIndex + 2; i < entities.size(); i++) {
                Entity e = entities.get(i);
                if (e instanceof VariantBegin) {
                    inVariantCount++;
                    if (inVariantCount == 1) {
                        if (entities.get(i + 1).equals(m)) {
                            selectedIndex = i + 1;
                            view.setSelectedIndex(i + 1);
                            return;
                        }
                    }
                } else if (e instanceof VariantEnd) {
                    inVariantCount--;
                } else if (inVariantCount == 0 && e instanceof Move || inVariantCount < 0) {
                    if (e.equals(m)) {
                        selectedIndex = i;
                        view.setSelectedIndex(selectedIndex);
                        return;
                    }
                    break;
                }
            }
        }

        // checks whether to insert new variant
        inVariantCount = 0;
        for (int i = selectedIndex + 1; i < entities.size(); i++) {
            Entity e = entities.get(i);
            if (e instanceof VariantBegin) {
                inVariantCount++;
            } else if (e instanceof VariantEnd) {
                if (inVariantCount == 0) {
                    position = i - 1;
                    break;
                } else {
                    inVariantCount--;
                }
            } else if (e instanceof Move && inVariantCount == 0) {
                position = i;
                createSubVariant = true;
                break;
            }
        }
        if (position < 0) {
            position = entities.size() > 0 ? entities.size() - 1 : 0;
        }
        if (createSubVariant) {
            entities.add(position + 1, new VariantBegin());
            entities.add(position + 2, m);
            entities.add(position + 3, new VariantEnd());
            selectedIndex = position + 2;
        } else {
            if (entities.size() == 0) {
                entities.add(position, m);
                selectedIndex = position;
            } else {
                entities.add(position + 1, m);
                selectedIndex = position + 1;
            }
        }
        view.setSelectedIndex(selectedIndex);
        view.notifyDataSetChanged();
    }


    @Override
    public void onCommentClicked(int position, Comment comment) {
        view.showEditComment(position, comment);
    }

    @Override
    public void onCommentAdded(String comment, int position) {
        entities.add(position, new Comment(comment));
        view.notifyItemInserted(position);
    }

    @Override
    public void onCommentEdited(String value, int position) {
        Comment comment = ((Comment) entities.get(position));
        comment.setValue(value);
        view.notifyItemChanged(position);
    }

    @Override
    public void removeComment(int position) {
        entities.remove(position);
        view.notifyItemRemoved(position);
    }

    @Override
    public void onMoveSelected(int position) {
        moveUpdatesLocked = true;
        ChessGame chessGame = new ChessGameImpl();
        List<Move> newMoves = MovesFilter.filter(entities, position);

        for (int i = 0; i < newMoves.size(); i++) {
            Move m = newMoves.get(i);
            chessGame.makeMove(m, m.getFromX(), m.getFromY(), m.getToX(), m.getToY());
        }
        this.chessGame.setChessGame(chessGame);

        moveUpdatesLocked = false;
    }

    @Override
    public boolean canDeleteMove(int position) {
        if (entities.size() == position + 1) return true;

        if (entities.size() > position + 1) {
            if (entities.get(position + 1) instanceof VariantEnd) {
                return true;
            }
        }
        return false;
    }

    @Override
    public void removeSubVariant(int position, Entity object) {
        int variantBeginPos = 0;
        int variantEndPos = 0;
        if (object instanceof VariantBegin) {
            int variantsCount = 1;
            variantBeginPos = position;
            variantEndPos = position;
            while (variantsCount > 0) {
                variantEndPos++;
                Entity e = entities.get(variantEndPos);
                if (e instanceof VariantBegin) {
                    variantsCount++;
                } else if (e instanceof VariantEnd) {
                    variantsCount--;
                }
            }
        } else if (object instanceof VariantEnd) {
            int variantsCount = 1;
            variantEndPos = position;
            variantBeginPos = position;
            while (variantsCount > 0) {
                variantBeginPos--;
                Entity e = entities.get(variantBeginPos);
                if (e instanceof VariantBegin) {
                    variantsCount--;
                } else if (e instanceof VariantEnd) {
                    variantsCount++;
                }
            }
        }

        for (int i = variantEndPos; i >= variantBeginPos; i--) {
            entities.remove(i);
            view.notifyItemRemoved(i);
        }
        if (selectedIndex != -1) {
            selectedIndex = -1;
            onMoveSelected(entities.size() - 1);
        }
    }

    @Override
    public void removeMove(int position) {
        if (!canDeleteMove(position)) return;

        entities.remove(position);
        view.notifyItemRemoved(position);
        if (position > 0) {
            Entity prev = entities.get(position - 1);
            if (prev instanceof VariantBegin) {
                entities.remove(position);
                view.notifyItemRemoved(position);
                entities.remove(position - 1);
                view.notifyItemRemoved(position - 1);
            }
        }

        selectedIndex = entities.size() - 1;
        onMoveSelected(entities.size() - 1);

    }

    @Override
    public void onSelectGlyphClicked(int position) {
        Entity e = entities.get(position);
        if (!(e instanceof Move)) {
            return;
        }
        view.showEditGlyph(position, ((Move) e).getNags());
    }

    @Override
    public void onGlyphChanged(int position, int[] nags) {
        Entity e = entities.get(position);
        if (!(e instanceof Move)) return;

        ((Move) e).setNags(nags);
        view.notifyItemChanged(position);
    }

    @Override
    public void onMoveClicked(int position) {
        if (position == selectedIndex) {
            onMoveSelected(entities.size() - 1);
            selectedIndex = entities.size() - 1;
            chessGame.setLocked(false);
        } else {
            onMoveSelected(position);
            selectedIndex = position;
        }
    }

    @Override
    public List<Entity> getMovesList() {
        return entities;
    }

    @Override
    public void setMovesList(List<Entity> e) {
        this.entities = e;
        view.setMovesList(e);
        view.notifyDataSetChanged();
        if (entities.size() > 0) {
            this.selectedIndex = 0;
            this.onMoveSelected(0);
        }
    }

    @Override
    public int getSelectedIndex() {
        return selectedIndex;
    }

    @Override
    public void setSelectedIndex(int index) {
        this.selectedIndex = index;
        onMoveSelected(index);
        view.setSelectedIndex(selectedIndex);
    }


}

