package chess.mobile.view.chess_view.providers;

import android.content.Context;
import android.content.res.AssetManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Log;
import android.util.SparseArray;

import java.io.IOException;
import java.io.InputStream;

import chess.mobile.chessgame.Consts;

/**
 * Created by Stanisław Kabaciński.
 */

public class AssetsBitmapProvider implements PieceBitmapProvider, Consts {
    public static final String TAG = "AssetsBitmapProvider";

    private Context context;
    private SparseArray<Bitmap> bitmapsCache;
    private BitmapFactory.Options options;
    private int size;

    public AssetsBitmapProvider(Context context, int size) {
        this.context = context;
        this.bitmapsCache = new SparseArray<>();
        this.size = size;
        preloadPiecesBitmaps();
    }

    private void preloadPiecesBitmaps() {
        try {
            calculateBitmapFactoryOptions();

            loadBitmap(BLACK_BISHOP, "b_bishop.png");
            loadBitmap(BLACK_KING, "b_king.png");
            loadBitmap(BLACK_KNIGHT, "b_knight.png");
            loadBitmap(BLACK_PAWN, "b_pawn.png");
            loadBitmap(BLACK_QUEEN, "b_queen.png");
            loadBitmap(BLACK_KS_ROOK, "b_rook.png");
            bitmapsCache.put(BLACK_QS_ROOK, bitmapsCache.get(BLACK_KS_ROOK));
            loadBitmap(WHITE_BISHOP, "w_bishop.png");
            loadBitmap(WHITE_KING, "w_king.png");
            loadBitmap(WHITE_KNIGHT, "w_knight.png");
            loadBitmap(WHITE_PAWN, "w_pawn.png");
            loadBitmap(WHITE_QUEEN, "w_queen.png");
            loadBitmap(WHITE_QS_ROOK, "w_rook.png");
            bitmapsCache.put(WHITE_KS_ROOK, bitmapsCache.get(WHITE_QS_ROOK));
        } catch (IOException e) {
            Log.e(TAG, "Failed to decode in bounds");
        }
    }

    private void calculateBitmapFactoryOptions() throws IOException {
        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inJustDecodeBounds = true;

        AssetManager assetManager = context.getAssets();
        InputStream istr;

        istr = assetManager.open("b_bishop.png");
        BitmapFactory.decodeStream(istr, null, options);
        int sampleSize = calculateInSampleSize(options, size, size);

        this.options = new BitmapFactory.Options();
        this.options.inJustDecodeBounds = false;
        this.options.inSampleSize = sampleSize;

        istr.close();
    }

    @Override
    public Bitmap getBitmapForPiece(int piece) {
        return bitmapsCache.get(piece);
    }

    private void loadBitmap(int piece, String path) throws IOException {
        bitmapsCache.put(piece, getBitmapFromAssets(path));
    }

    private Bitmap getBitmapFromAssets(String filename) throws IOException {
        AssetManager assetManager = context.getAssets();
        InputStream istr = assetManager.open(filename);
        Bitmap bitmap = BitmapFactory.decodeStream(istr, null, options);
        if (istr != null) {
            istr.close();
        }
        return Bitmap.createScaledBitmap(bitmap, size, size, false);
    }

    private int calculateInSampleSize(
            BitmapFactory.Options options, int reqWidth, int reqHeight) {
        final int height = options.outHeight;
        final int width = options.outWidth;
        int inSampleSize = 1;

        if (height > reqHeight || width > reqWidth) {

            final int halfHeight = height / 2;
            final int halfWidth = width / 2;

            while ((halfHeight / inSampleSize) >= reqHeight
                    && (halfWidth / inSampleSize) >= reqWidth) {
                inSampleSize *= 2;
            }
        }

        return inSampleSize;
    }
}
