package chess.mobile.view.chess_view.renderers;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Rect;
import android.util.Log;
import android.view.SurfaceHolder;

import java.util.Locale;

import chess.mobile.view.chess_view.providers.AssetsBitmapProvider;

/**
 * Created by Stanisław Kabaciński.
 */

public class ChessboardRenderer implements SurfaceHolder.Callback {
    public static final String TAG = "ChessboardRenderer";
    public static final boolean DEBUG = true;

    private BoardRenderer boardRenderer;
    private PieceRenderer pieceRenderer;
    private OverlayRenderer overlayRenderer;

    private Context context;
    private SurfaceHolder holder;
    private CanvasUpdateRunner runner;

    private float height;
    private float width;
    private float aspect;
    private int[][] board;

    private boolean inverted;

    public ChessboardRenderer(Context context) {
        this.context = context;
    }

    public void onDraw(Canvas canvas) {
        boardRenderer.draw(canvas);
        pieceRenderer.draw(canvas);
        overlayRenderer.draw(canvas);
    }

    @Override
    public void surfaceCreated(SurfaceHolder holder) {
        Rect rect = holder.getSurfaceFrame();
        this.width = rect.width();
        this.height = rect.height();
        this.aspect = width / height;
        boardRenderer = new BoardRenderer(this);
        pieceRenderer = new PieceRenderer(
                new AssetsBitmapProvider(context, (int) boardRenderer.getFieldSize()),
                boardRenderer, this);
        pieceRenderer.setBoard(board);
        overlayRenderer = new OverlayRenderer(boardRenderer, this);
        log(String.format(Locale.getDefault(),
                "Surface created - width=%f, height=%f, aspect=%f", width, height, aspect));
        this.holder = holder;

        if (runner != null && runner.isRunning()) {
            try {
                runner.setRunning(false);
                runner.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        runner = new CanvasUpdateRunner();
        runner.setRunning(true);
        runner.start();
    }

    @Override
    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
        this.width = width;
        this.height = height;
        this.aspect = width / height;
        boardRenderer.updateDimensions();
    }

    @Override
    public void surfaceDestroyed(SurfaceHolder holder) {
        runner.setRunning(false);
        try {
            runner.join();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    private void log(String msg) {
        if (!DEBUG) return;
        Log.d(TAG, msg);
    }

    public float getHeight() {
        return height;
    }

    public float getWidth() {
        return width;
    }

    public float getAspect() {
        return aspect;
    }

    public int[][] getBoard() {
        return board;
    }

    public void setBoard(int[][] board) {
        this.board = board;
        if (pieceRenderer != null) pieceRenderer.setBoard(board);
    }

    public float getCenterX() {
        return boardRenderer.getCenterX();
    }

    public float getCenterY() {
        return boardRenderer.getCenterY();
    }

    public float getFieldSize() {
        return boardRenderer.getFieldSize();
    }

    public OverlayRenderer getOverlayRenderer() {
        return overlayRenderer;
    }

    public boolean isInverted() {
        return inverted;
    }

    public void setInverted(boolean inverted) {
        this.inverted = inverted;
    }

    private class CanvasUpdateRunner extends Thread {

        private boolean running;

        @Override
        public void run() {
            while (running) {
                Canvas canvas = holder.lockCanvas(null);

                if (canvas != null) {
                    synchronized (holder) {
                        onDraw(canvas);
                    }
                    holder.unlockCanvasAndPost(canvas);
                }
                try {
                    Thread.sleep(32);
                } catch (InterruptedException ex) {
                    // ignore
                }
            }
        }

        public boolean isRunning() {
            return running;
        }

        public void setRunning(boolean running) {
            this.running = running;
        }
    }
}
