package chess.mobile.wsclient;

import android.app.Service;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.IBinder;
import android.os.Looper;
import android.os.Message;
import android.support.annotation.Nullable;
import android.util.Log;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;

import chess.mobile.dao.UserSettingsDAOImpl;
import chess.mobile.event.ConnectToWebSocketEvent;
import chess.mobile.event.DisconnectFromWebSocketEvent;
import chess.mobile.event.MessageReceivedEvent;
import chess.mobile.event.SendMessageEvent;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.WebSocket;

/**
 * Created by Stanisław Kabaciński.
 */

public class WebSocketClient extends Service {

    private static final String TAG = "WebSocketClient";

    private static final int CONNECT_TO_WEB_SOCKET = 1;
    private static final int SEND_MESSAGE = 2;
    private static final int CLOSE_WEB_SOCKET = 3;
    private static final int DISCONNECT_LOOPER = 4;

    private static final String KEY_MESSAGE = "message";
    private static final String KEY_URL = "url";

    private Handler serviceHandler;
    private Looper serviceLooper;
    private WebSocket webSocket;
    private boolean connected;

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    private final class ServiceHandler extends Handler {

        public ServiceHandler(Looper looper) {
            super(looper);
        }

        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case CONNECT_TO_WEB_SOCKET:
                    connectToWebSocket(msg.getData().getString(KEY_URL));
                    break;
                case SEND_MESSAGE:
                    sendMessageThroughWebSocket(msg.getData().getString(KEY_MESSAGE));
                    break;
                case CLOSE_WEB_SOCKET:
                    closeWebSocket();
                    break;
                case DISCONNECT_LOOPER:
                    serviceLooper.quit();
                    break;
            }
        }
    }

    @Subscribe
    @SuppressWarnings("unused")
    public void onDisconnectEvent(DisconnectFromWebSocketEvent event) {
        if (connected) {
            Message message = Message.obtain();
            message.what = CLOSE_WEB_SOCKET;
            serviceHandler.sendMessage(message);
        } else {
            Log.d(TAG, "Cannot disconnect from WebSocket, already disconnected.");
        }
    }

    @Subscribe
    @SuppressWarnings("unused")
    public void onSendMessageEvent(SendMessageEvent sendMessageEvent) {
        if (connected) {
            Message message = Message.obtain();
            message.what = SEND_MESSAGE;
            Bundle data = new Bundle();
            data.putString(KEY_MESSAGE, sendMessageEvent.message);
            message.setData(data);
            serviceHandler.sendMessage(message);
        } else {
            Log.d(TAG, "Can`t send message \"" + sendMessageEvent.message + "\". Client not connected.");
        }
    }

    @Subscribe
    @SuppressWarnings("unused")
    public void onConnectToWebSocketEvent(ConnectToWebSocketEvent event) {
        if (connected) {
            serviceHandler.sendEmptyMessage(CLOSE_WEB_SOCKET);
        }
        Message m = Message.obtain();
        m.what = CONNECT_TO_WEB_SOCKET;
        Bundle data = new Bundle();
        data.putString(KEY_URL, event.url);
        m.setData(data);
        serviceHandler.sendMessage(m);
    }

    private void sendMessageThroughWebSocket(String message) {
        if (!connected) {
            return;
        }
        if (!webSocket.send(message)) {
            Log.e(TAG, "Failed to send message");
        } else {
            Log.e(TAG, "Message \"" + message + "\" send.");
        }
    }

    private void connectToWebSocket(String url) {
        OkHttpClient client = new OkHttpClient();
        Request request = new Request.Builder()
                .addHeader("Authorization", "Bearer " + new UserSettingsDAOImpl(this).getAuthToken())
                .url(url)
                .build();
        webSocket = client.newWebSocket(request, new WebSocketListener());
    }

    private void closeWebSocket() {
        if (!connected) {
            return;
        }
        if (!webSocket.close(1000, "Socket closed.")) {
            Log.d(TAG, "Failed to close WebSocket");
        }
        Log.d(TAG, "Session closed");
    }

    @Override
    public void onCreate() {
        super.onCreate();
        HandlerThread thread = new HandlerThread("WebSocket service");
        thread.start();
        serviceLooper = thread.getLooper();
        serviceHandler = new ServiceHandler(serviceLooper);
        EventBus.getDefault().register(this);
    }

    @Override
    public void onDestroy() {
        serviceHandler.sendEmptyMessage(CLOSE_WEB_SOCKET);
        serviceHandler.sendEmptyMessage(DISCONNECT_LOOPER);
        EventBus.getDefault().unregister(this);
        super.onDestroy();
    }

    private final class WebSocketListener extends okhttp3.WebSocketListener {
        @Override
        public void onOpen(WebSocket webSocket, Response response) {
            super.onOpen(webSocket, response);
            connected = true;
            Log.d(TAG, "Web socket opened");
        }

        @Override
        public void onMessage(WebSocket webSocket, String text) {
            super.onMessage(webSocket, text);
            Log.d(TAG, "Message received " + text);
            EventBus.getDefault().post(new MessageReceivedEvent(text));
        }

        @Override
        public void onClosed(WebSocket webSocket, int code, String reason) {
            connected = false;
            Log.d(TAG, "Web socket closed reason: " + reason);
        }

        @Override
        public void onFailure(WebSocket webSocket, Throwable t, Response response) {
            connected = false;
            Log.d(TAG, "Web socket failed ", t);
        }
    }
}
